#include <iostream>
#include <string>
#include "ipglasma.hpp"
#include <cmath>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <omp.h>

using namespace std;

int printDipoleAmplitudes(string fname, IPGlasma ipglasma)
{
    //ofstream output0;
    //Initialize params
    double q1[2] = {0,0};
    double q2[2] = {0,0};
    double r = 0;
    double dipoleAmp = 0;
    //TODO: set number of different rotations for dipole!
    int N = 100;

    cout << fixed << setprecision(8);
    
    cout << "# Dipole amplitudes for b=0"<< endl;
    cout << "# " << N << " rotations " << endl;
    cout << "# Filename " << fname << endl;
    cout << "# -------------------------"<< endl;
    cout << "# r, D_p(r,0)" << endl;

    // loop to calculate dipole amplitude at radii r=j*10^i
    for (int i=-4; i < 2; i += 1){
        cout <<""<< endl;

        // use index j to get radius
        for (double j=1; j < 10; j += 1){
            r = j*pow(10, i);
            // go through different orientations of the dipole via rotations
            for (double k=0; k < N; k += 1){
            // inside cos/sin is k*1/N*2*3.141592
                q1[0] = 0.5*r*cos(k*1./N*2*3.141592);
                q1[1] = 0.5*r*sin(k*1./N*2*3.141592);
                q2[0] = -q1[0];
                q2[1] = -q1[1];
    
                dipoleAmp += ipglasma.Amplitude(0, q1, q2);
            }

            dipoleAmp = dipoleAmp/N;
            cout << r << ", " << dipoleAmp << endl;
            dipoleAmp = 0;
        }
    }

    return 0; 
}

        int thread_id = omp_get_thread_num();
        int num_threads = omp_get_num_threads();
    
        // Print the number of threads (only from the master thread to avoid too many prints)
        if (thread_id == 0) {
            cout << "# Number of threads: " << num_threads << endl;
        }

        // Print the thread id for debugging (can be limited to a few threads for easier reading)
        if (thread_id < 5) {  // Change 5 to a larger number if needed
             cout << "# Thread " << thread_id << " is starting" << endl;
        }

    } 
    cout << "# Hello from main.cpp sumAmplitudes!" << endl;
    double sum = 0.0;
    int L = 512; //ipglasma->xcoords.size();
    cout << "# Summing with lattice size L = " << L << endl;
    //WilsonLine quark;
    //int thrownAway = 0;
    //int iWasHere = 0;
    #pragma omp parallel for reduction(+:sum)//,thrownAway,iWasHere) //collapse(4)

    for(int i =0; i < L; i += 1){ 
        for(int j =0; j < L; j += 1){ 
    
            //#pragma omp parallel for reduction(+:sum) collapse(2)
            for(int k =0; k < L; k +=1){
//                cout << "# Thread " << omp_get_thread_num() << " is processing (" << i << ", " << j << ", " << k <<")" << endl;
                for(int l =0; l < L; l +=1){
                    int q1[2] = {i,j};
                    double r1 = ipglasma->DistanceToOrigin(q1);
                    int q2[2] = {k,l};
                    //double r1 = ipglasma->DistanceToOrigin(q1);
                    double r2 = ipglasma->DistanceToOrigin(q2);
                    if (r1 > 1. && r2 > 1.){
                       // cout << "#!!Throwing away large dipole separation!" << endl;
                        //cout << "#Quark coordinate indices " << q1[0] << "," << q1[1] << " and " << q2[0] << "," << q2[1] << endl;
                        //cout << "#Quark coordinates " << X(q1[0]) << "," << Y(q1[1]) << " and " << X(q2[0]) << "," << Y(q2[1]) << endl;
                        //cout << "#Dipole size is " << r1-r2 << endl;
                        //cout << "#Dipole amplitude is " << Amplitude(0, q1, q2) << endl;
                        //thrownAway += 1;
                        continue;
                }

                    // Haw w2 = WilsonLine(k,l)
                    // Laske w1.MultiplbyHermitianConjugate(w2);

                   sum += ipglasma->Amplitude(0, q1, q2);
                   //sum += Amplitude(0, quark, q2);
                   //iWasHere += 1;  
                }
            }
        }
    }   
    cout << "# total sum = " << sum << endl;
    //cout << "# i counted " << iWasHere << " values" << endl;
    //cout << "# threw away " << thrownAway << " values (" << 100.*thrownAway*pow(L,-4) << "%)" << endl;
    return sum;
}

int main(int argc, char* argv[])
{
    if (argc < 3 || argc > 4){
        cerr << "Usage: " << argv[0] << " filename 1(integrate)/0(print amplitudes) [task_id]" << endl;
    }
    
    string fname = argv[1];
    IPGlasma ipglasma(fname, 0, BINARY);

    int task_id = 0;

    if (argc > 3){ 
        task_id= atoi(argv[3]);
        cout << "#Integrating array job #" << task_id << endl;
        ipglasma->SumAmplitudes(task_id);   
    }

    else if (!argv[2]){ 
        printDipoleAmplitudes(fname, ipglasma);
    }
    
    else{ 
        double sum = ipglasma->SumAmplitudes();
        // Save results
        //std::filesystem::create_directory("output_"+fname);
        ofstream outfile("da_sum_ +" + fname + ".txt");
        outfile << sum << endl;
	outfile.close();
    }

    return 0;
}   
